@@ -3,45 +3,25 @@ use strict;
 use warnings;
 package YAML::PP::Constructor;
 
-our $VERSION = '0.004_002'; # TRIAL VERSION
+our $VERSION = '0.005_001'; # TRIAL VERSION
+
+use YAML::PP;
 
 use constant DEBUG => ($ENV{YAML_PP_LOAD_DEBUG} or $ENV{YAML_PP_LOAD_TRACE}) ? 1 : 0;
 use constant TRACE => $ENV{YAML_PP_LOAD_TRACE} ? 1 : 0;
 
-my $RE_INT = '[+-]?[1-9]\d*';
-my $RE_OCT = '0o[1-7][0-7]*';
-my $RE_HEX = '0x[1-9a-fA-F][0-9a-fA-F]*';
-my $RE_FLOAT = '[+-]?(?:\.\d+|\d+\.\d*)(?:[eE][+-]?\d+)?';
-my $RE_NUMBER ="'(?:$RE_INT|$RE_OCT|$RE_HEX|$RE_FLOAT)";
-
 
 sub new {
     my ($class, %args) = @_;
 
-    my $bool = delete $args{boolean} // 'perl';
-    my $truefalse;
-    if ($bool eq 'JSON::PP') {
-        require JSON::PP;
-        $truefalse = \&bool_jsonpp;
-    }
-    elsif ($bool eq 'boolean') {
-        require boolean;
-        $truefalse = \&bool_booleanpm;
-    }
-    elsif ($bool eq 'perl') {
-        $truefalse = \&bool_perl;
-    }
-    else {
-        die "Invalid value for 'boolean': '$bool'. Allowed: ('perl', 'boolean', 'JSON::PP')";
-    }
+    my $schema = delete $args{schema};
 
     if (keys %args) {
         die "Unexpected arguments: " . join ', ', sort keys %args;
     }
 
     my $self = bless {
-        boolean => $bool,
-        truefalse => $truefalse,
+        schema => $schema,
     }, $class;
 }
 
@@ -61,8 +41,7 @@ sub set_data { $_[0]->{data} = $_[1] }
 sub set_docs { $_[0]->{docs} = $_[1] }
 sub set_refs { $_[0]->{refs} = $_[1] }
 sub set_anchors { $_[0]->{anchors} = $_[1] }
-sub boolean { return $_[0]->{boolean} }
-sub truefalse { return $_[0]->{truefalse} }
+sub schema { return $_[0]->{schema} }
 
 sub begin {
     my ($self, $data, $event) = @_;
@@ -159,12 +138,18 @@ sub end {
 
 sub scalar_event {
     my ($self, $event) = @_;
-    my $value = $self->render_value($event);
+    DEBUG and warn "CONTENT $event->{value} ($event->{style})\n";
+    my $value;
+    if ($event->{tag}) {
+        $value = $self->schema->load_scalar_tag($event);
+    }
+    else {
+        $value = $self->schema->load_scalar($event->{style}, $event->{value});
+    }
     if (defined (my $name = $event->{anchor})) {
         $self->anchors->{ $name } = \$value;
     }
-    $self->event(value => $value, event => $event);
-    DEBUG and warn YAML::PP::Parser->event_to_test_suite([value => $event]) ."\n";
+    $self->add_scalar($value);
 }
 
 sub alias_event {
@@ -174,14 +159,11 @@ sub alias_event {
     if (my $anchor = $self->anchors->{ $name }) {
         $value = $$anchor;
     }
-    DEBUG and warn YAML::PP::Parser->event_to_test_suite([alias => $event]) ."\n";
-    $self->event(value => $value, event => $event);
+    $self->add_scalar($value);
 }
 
-sub event {
-    my ($self, %args) = @_;
-    my $value = $args{value};
-    my $event = $args{event};
+sub add_scalar {
+    my ($self, $value) = @_;
 
     my $refs = $self->refs;
 
@@ -200,23 +182,6 @@ sub event {
     }
 }
 
-sub render_value {
-    my ($self, $info) = @_;
-    my $value;
-    my $content = $info->{value};
-    my $style = $info->{style};
-    DEBUG and warn "CONTENT $content ($style)\n";
-    if ($style eq ':') {
-        $value = $self->render_plain_scalar($content);
-    }
-    else {
-        $value = $content;
-    }
-    TRACE and local $Data::Dumper::Useqq = 1;
-    TRACE and warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$value], ['value']);
-    return $value;
-}
-
 sub stringify_complex {
     my ($self, $data) = @_;
     require Data::Dumper;
@@ -230,40 +195,4 @@ sub stringify_complex {
     return $string;
 }
 
-sub render_plain_scalar {
-    my ($self, $content) = @_;
-    return unless defined $content;
-    my $value;
-    if ($content =~ m/^($RE_INT|$RE_FLOAT)$/){
-        $value = 0 + $1;
-    }
-    elsif ($content =~ m/^($RE_HEX)/) {
-        $value = hex $content;
-    }
-    elsif ($content =~ m/^($RE_OCT)/) {
-        my $oct = 0 . substr($content, 2);
-        $value = oct $oct;
-    }
-    elsif ($content eq 'true' or $content eq 'false') {
-        $value = $self->truefalse->($content);
-    }
-    else {
-        $value = $content;
-    }
-    return $value;
-}
-
-sub bool_jsonpp {
-    $_[0] eq 'true' ? JSON::PP::true() : JSON::PP::false()
-}
-
-sub bool_booleanpm {
-    $_[0] eq 'true' ? boolean::true() : boolean::false()
-}
-
-sub bool_perl {
-    $_[0] eq 'true' ? 1 : 0
-}
-
-
 1;
